
#include "xv.h"

#define GetByte(file)	getc(file)

static int	GetWord(FILE *file)
{
	int	lo,hi;

	return ((lo=GetByte(file))==EOF ||
		(hi=GetByte(file))==EOF)?-1:(hi<<8)|lo;
}

#define Abort(msg)	{ \
	SetISTR(ISTR_WARNING,"%s:  %s",baseName,msg); return 0; \
}
#define Error(msg)	{ \
	(void)fclose(file); Abort(msg); \
}

static int	sameFile=0;
static char	lastPath[MAXPATHLEN+1]="";

int	LoadHiPic(char *path,PICINFO *pi)
{
	FILE	*file;
	char	*baseName=BaseName(path),*comment=NULL;
	int	len,w,h,idx,size,data,base,roof,shift;
	u_short	*tmpData;
	byte	*picData;
	double	step;

	pi->pic=NULL;
	pi->comment=NULL;

	if ((file=xv_fopen(path,"r"))==NULL) Abort("can't open file");

	if (GetByte(file)!='I' || GetByte(file)!='M') Error("unknown format");

	if ((len=GetWord(file))<0) Error("bad comment length");

	if ((w=GetWord(file))<=0 ||
	    (h=GetWord(file))<=0) Error("bad image size");

	(void)fseek(file,4L,1);		/* image offset */

	if (GetWord(file)!=2) Error("bad image type");

	(void)fseek(file,50L,1);	/* reserved data */

	if (len) {
	    if ((comment=(char *)malloc(len+1))==NULL)
		Error("malloc failure for comment");

	    for (idx=0; idx<len; idx++) comment[idx]=GetByte(file);

	    comment[len]='\0';
	}
	size=w*h;

	if ((tmpData=(u_short *)malloc(size*sizeof(*tmpData)))==NULL) {
	    free(comment); Error("malloc failure for image data");
	}
	for (idx=0; idx<size; idx++) {
	    if ((data=GetWord(file))<0) {
		free(tmpData); free(comment); Error("premature EOF");
	    }
	    tmpData[idx]=(u_short)data;

	    if (idx==0)
		base=roof=data;
	    else
		if (data<base) base=data; else if (data>roof) roof=data;
	}
	(void)fclose(file);

	if (strcmp(lastPath,path)) {
	    sameFile=0; (void)strcpy(lastPath,path);
	}
	else
	    sameFile=(!sameFile);

	picData=(byte *)tmpData;
	if (sameFile) {
	    step=(base==roof)?1.0:(double)(roof-base)/255.0;
	    for (idx=0; idx<size; idx++)
		picData[idx]=(byte)((double)(tmpData[idx]-base)/step+0.5);

	    (void)sprintf(pi->fullInfo,
			  "HiPic, type 2 format, %d ~ %d. (%d bytes)",
			  base,roof,64+len+size*2);
	}
	else {
	    for (shift=0; roof>>shift>=256; shift++) ;

	    for (idx=0; idx<size; idx++) picData[idx]=tmpData[idx]>>shift;

	    (void)sprintf(pi->fullInfo,
			  "HiPic, type 2 format, %d bits. (%d bytes)",
			  shift+8,64+len+size*2);
	}
	if ((pi->pic=(byte *)realloc(tmpData,size))==NULL) pi->pic=picData;

	pi->w=pi->normw=w;
	pi->h=pi->normh=h;
	pi->type=PIC8;

	for (idx=0; idx<256; idx++) pi->r[idx]=pi->g[idx]=pi->b[idx]=idx;

	pi->frmType=(-1);
	pi->colType=F_GREYSCALE;

	(void)sprintf(pi->shrtInfo,"%dx%d HiPic.",w,h);

	pi->comment=comment;
	pi->numpages=1;

	return 1;
}
